@@ -22,7 +22,9 @@ from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
 VOLUME_ID = 'f10d1a84-9b7b-427e-8fec-63c48b509a56'
 LUN_ID = 'ee6b4cc7-477b-4016-aa0c-7127b4e3af86'
 LUN_HANDLE = 'fake_lun_handle'
+NAMESPACE_HANDLE = 'fake_namespace_handle'
 LUN_NAME = 'lun1'
+NAMESPACE_NAME = 'namespace1'
 LUN_SIZE = 3
 LUN_TABLE = {LUN_NAME: None}
 SIZE = 1024
@@ -40,6 +42,7 @@ AGGREGATE = 'aggr1'
 FLEXVOL = 'openstack-flexvol'
 NFS_FILE_PATH = 'nfsvol'
 PATH = '/vol/%s/%s' % (POOL_NAME, LUN_NAME)
+PATH_NAMESPACE = '/vol/%s/%s' % (POOL_NAME, NAMESPACE_NAME)
 IMAGE_FILE_ID = 'img-cache-imgid'
 PROVIDER_LOCATION = 'fake_provider_location'
 NFS_HOST = 'nfs-host1'
@@ -61,10 +64,25 @@ JOB_UUID = 'fb132b04-6422-43ce-9451-ee819f0131a4'
 LUN_METADATA = {
     'OsType': None,
     'SpaceReserved': 'true',
+    'SpaceAllocated': 'false',
     'Path': PATH,
     'Qtree': None,
     'Volume': POOL_NAME,
 }
+LUN_METADATA_WITH_SPACE_ALLOCATION = {
+    'OsType': None,
+    'SpaceReserved': 'true',
+    'Path': PATH,
+    'SpaceAllocated': 'true',
+    'Qtree': None,
+    'Volume': POOL_NAME,
+}
+NAMESPACE_METADATA = {
+    'OsType': None,
+    'Path': PATH_NAMESPACE,
+    'Qtree': None,
+    'Volume': POOL_NAME,
+}
 VOLUME = {
     'name': LUN_NAME,
     'size': SIZE,
@@ -72,6 +90,13 @@ VOLUME = {
     'host': HOST_STRING,
     'attach_status': DETACHED,
 }
+NAMESPACE_VOLUME = {
+    'name': NAMESPACE_NAME,
+    'size': SIZE,
+    'id': VOLUME_ID,
+    'host': HOST_STRING,
+    'attach_status': DETACHED,
+}
 NFS_VOLUME = {
     'name': NFS_FILE_PATH,
     'size': SIZE,
@@ -223,6 +248,7 @@ ISCSI_VOLUME = {
     'name': 'fake_volume',
     'id': 'fake_id',
     'provider_auth': 'fake provider auth',
+    'provider_location': 'iscsi:/dummy_path'
 }
 
 ISCSI_LUN = {'name': ISCSI_VOLUME, 'lun_id': 42}
@@ -234,6 +260,7 @@ ISCSI_CONNECTION_PROPERTIES = {
         'auth_method': 'fake_method',
         'auth_password': 'auth',
         'auth_username': 'provider',
+        'discard': True,
         'discovery_auth_method': 'fake_method',
         'discovery_auth_username': 'provider',
         'discovery_auth_password': 'auth',
@@ -273,7 +300,7 @@ IGROUP1 = {'initiator-group-os-type': 'linux',
            'initiator-group-name': IGROUP1_NAME}
 
 QOS_SPECS = {}
-EXTRA_SPECS = {}
+EXTRA_SPECS = {'netapp:space_allocation': '<is> True'}
 MAX_THROUGHPUT = '21734278B/s'
 MIN_IOPS = '256iops'
 MAX_IOPS = '512iops'
@@ -349,6 +376,8 @@ VOLUME_PATH = '/vol/%s/%s' % (NETAPP_VOLUME, VOLUME_NAME)
 MOUNT_PATH = '168.10.16.11:/' + VOLUME_ID
 SNAPSHOT_NAME = 'fake_snapshot_name'
 SNAPSHOT_LUN_HANDLE = 'fake_snapshot_lun_handle'
+SNAPSHOT_NAMESPACE_HANDLE = 'fake_snapshot_namespace_handle'
+
 SNAPSHOT_MOUNT = '/fake/mount/path'
 
 SNAPSHOT = {
@@ -381,6 +410,21 @@ LUN_WITH_METADATA = {
     }
 }
 
+NAMESPACE_WITH_METADATA = {
+    'handle': 'vserver_fake:/vol/fake_flexvol/volume-fake-uuid',
+    'name': 'volume-fake-uuid',
+    'size': 20971520,
+    'metadata': {
+        'Vserver': 'vserver_fake',
+        'Volume': 'fake_flexvol',
+        'Qtree': None,
+        'Path': '/vol/fake_flexvol/volume-fake-uuid',
+        'OsType': 'linux',
+        'SpaceReserved': 'false',
+        'UUID': 'fake-uuid'
+    }
+}
+
 VOLUME_REF = {'name': 'fake_vref_name', 'size': 42}
 
 FAKE_CMODE_VOLUMES = ['open123', 'mixed', 'open321']
@@ -723,6 +767,22 @@ test_volume.state = {
 test_volume.qos = {'qos_policy_group': None}
 test_volume.host = 'fakehost@backbackend#fakepool'
 test_volume.name = 'fakename'
+test_volume.size = SIZE
+test_volume.multiattach = False
+
+
+class test_namespace_volume(object):
+
+    def __getitem__(self, key):
+        return getattr(self, key)
+
+
+test_namespace_volume = test_namespace_volume()
+test_namespace_volume.name = NAMESPACE_NAME
+test_namespace_volume.size = SIZE
+test_namespace_volume.id = VOLUME_ID
+test_namespace_volume.host = HOST_STRING
+test_namespace_volume.attach_status = DETACHED
 
 
 class test_snapshot(object):
@@ -975,3 +1035,6 @@ ADAPTIVE_QOS_POLICY_GROUP_INFO_REST = {
 }
 
 REST_FIELDS = 'uuid,name,style'
+SUBSYSTEM = 'openstack-fake-subsystem'
+HOST_NQN = 'nqn.1992-01.example.com:string'
+TARGET_NQN = 'nqn.1992-01.example.com:target'
